/*
 * Created on: 07-Dec-2005
 * Author: Dibyendu Majumdar
 */
package org.simpledbm.samples.latch;

import org.simpledbm.common.latch.Latch;
import org.simpledbm.common.latch.LatchFactory;
import org.simpledbm.common.latch.impl.LatchFactoryImpl;

/**
 * Demonstrates how to obtain Latches and use them.
 *
 * @author Dibyendu Majumdar
 * @since 07-Dec-2005
 */
public class LatchDemo {

    public static void main(String[] args) {

        /*
         * Obtain an instance of the Latch Factory.
         * This should preferably use dependency injection
         * pattern.
         */
        LatchFactory latchFactory = new LatchFactoryImpl();
        
        /*
         * Create a new instance of ReadWriteUpdate latch.
         */
        Latch latch = latchFactory.newReadWriteUpdateLatch();
        
        /*
         * Acquire an update mode lock.
         */
        latch.updateLock();
        
        /*
         * Upgrade lock to exclusive mode.
         */
        latch.upgradeUpdateLock();
        
        /*
         * Downgrade lock to update mode.
         */
        latch.downgradeExclusiveLock();
        
        /*
         * Downgrade lock to shared mode.
         */
        latch.downgradeUpdateLock();
        
        /*
         * Unlock; note that this requires knowledge of current
         * lock mode.
         */
        latch.unlockShared();
    }

}
