/***
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Project: www.simpledbm.org
 *    Author : Dibyendu Majumdar
 *    Email  : dibyendu@mazumdar.demon.co.uk
 */
/*
 * Created on: 06-Dec-2005
 * Author: Dibyendu Majumdar
 */
package org.simpledbm.samples.transaction;

import org.simpledbm.common.latch.LatchFactory;
import org.simpledbm.common.latch.impl.LatchFactoryImpl;
import org.simpledbm.common.registry.ObjectFactory;
import org.simpledbm.common.registry.impl.ObjectFactoryImpl;
import org.simpledbm.rss.bm.BufMgr;
import org.simpledbm.rss.bm.BufMgrException;
import org.simpledbm.rss.bm.impl.BufMgrImpl;
import org.simpledbm.rss.io.StorageContainer;
import org.simpledbm.rss.io.StorageContainerFactory;
import org.simpledbm.rss.io.StorageException;
import org.simpledbm.rss.io.StorageManager;
import org.simpledbm.rss.io.impl.FileStorageContainerFactory;
import org.simpledbm.rss.io.impl.StorageManagerImpl;
import org.simpledbm.rss.locking.LockMgr;
import org.simpledbm.rss.locking.LockMgrFactory;
import org.simpledbm.rss.locking.impl.LockMgrFactoryImpl;
import org.simpledbm.rss.log.LogException;
import org.simpledbm.rss.log.LogMgr;
import org.simpledbm.rss.log.impl.LogFactoryImpl;
import org.simpledbm.rss.pm.Page;
import org.simpledbm.rss.pm.PageException;
import org.simpledbm.rss.pm.PageFactory;
import org.simpledbm.rss.pm.PageId;
import org.simpledbm.rss.pm.impl.PageFactoryImpl;
import org.simpledbm.rss.sp.SlottedPageMgr;
import org.simpledbm.rss.sp.impl.SlottedPageMgrImpl;
import org.simpledbm.rss.tm.LoggableFactory;
import org.simpledbm.rss.tm.TransactionalModuleRegistry;
import org.simpledbm.rss.tm.TrxException;
import org.simpledbm.rss.tm.TrxMgr;
import org.simpledbm.rss.tm.impl.LoggableFactoryImpl;
import org.simpledbm.rss.tm.impl.TransactionalModuleRegistryImpl;
import org.simpledbm.rss.tm.impl.TrxMgrImpl;

public class OneBitMgrDatabase {

    final LogFactoryImpl logFactory = new LogFactoryImpl();

    final ObjectFactory objectFactory = new ObjectFactoryImpl();

    final StorageContainerFactory storageFactory = new FileStorageContainerFactory();

    final StorageManager storageManager = new StorageManagerImpl();

    final LatchFactory latchFactory = new LatchFactoryImpl();

    final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
            storageManager, latchFactory);

    final SlottedPageMgr spmgr = new SlottedPageMgrImpl(objectFactory);

    final LoggableFactory loggableFactory = new LoggableFactoryImpl(
            objectFactory);

    final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();

    final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();

    final LockMgr lockmgr = lockmgrFactory.create(null);

    LogMgr logmgr;

    BufMgr bufmgr;

    TrxMgr trxmgr;

    public OneBitMgrDatabase() {
        super();
    }

    public void create() throws LogException, StorageException, PageException,
            TrxException, BufMgrException {
        try {
            logFactory.createLog(null);

            logmgr = logFactory.getLog(null);
            logmgr.start();
            StorageContainer sc = storageFactory.create("dual");
            storageManager.register(0, sc);
            Page page = pageFactory.getInstance(pageFactory.getRawPageType(),
                    new PageId(0, 0));
            pageFactory.store(page);
            bufmgr = new BufMgrImpl(logmgr, pageFactory, 5, 11);
            bufmgr.start();
            trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager,
                    bufmgr, lockmgr, loggableFactory, latchFactory,
                    objectFactory, moduleRegistry);
            trxmgr.start();
        } finally {
            shutdown();
        }
    }

    public void open() throws LogException, TrxException, BufMgrException {
        logmgr = logFactory.getLog(null);
        logmgr.start();
        bufmgr = new BufMgrImpl(logmgr, pageFactory, 5, 11);
        bufmgr.start();
        trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr,
                lockmgr, loggableFactory, latchFactory, objectFactory,
                moduleRegistry);
        trxmgr.start();
    }

    public OneBitMgr getOneBitMgr() {
        return new OneBitMgr(storageFactory, storageManager, pageFactory, bufmgr, loggableFactory, trxmgr, objectFactory, moduleRegistry);
    }
    
    /**
     * Shuts down the database components.
     */
    public void shutdown() {
    	trxmgr.shutdown();
        bufmgr.shutdown();
        logmgr.shutdown();
        storageManager.shutdown();
    }

    public final TrxMgr getTrxmgr() {
        return trxmgr;
    }
    
    
    
}
