/***
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Project: www.simpledbm.org
 *    Author : Dibyendu Majumdar
 *    Email  : dibyendu@mazumdar.demon.co.uk
 */
/*
 * Created on: Nov 14, 2005
 * Author: Dibyendu Majumdar
 */
package org.simpledbm.samples.typesystem;

import java.nio.ByteBuffer;


abstract class BaseField implements Field {

    private static final int NULL_FIELD = 1;
    private static final int MINUS_INFINITY_FIELD = 2;
    private static final int VALUE_FIELD = 4;
    private static final int PLUS_INFINITY_FIELD = 8;
    
    private byte statusByte = 0; 
    private TypeDescriptor typeDesc;
    
    protected BaseField(TypeDescriptor typeDesc) {
        statusByte = NULL_FIELD;
        this.typeDesc = typeDesc;
    }
    
    public int getInt() {
        throw new UnsupportedOperationException();
    }

    public void setInt(Integer integer) {
        throw new UnsupportedOperationException();
    }

    public String getString() {
        throw new UnsupportedOperationException();
    }

    public void setString(String string) {
        throw new UnsupportedOperationException();
    }

    public void retrieve(ByteBuffer bb) {
        statusByte = bb.get();
    }

    public void store(ByteBuffer bb) {
        bb.put(statusByte);
    }

    public int getStoredLength() {
        return 1;
    }

    public int compareTo(Field other) {
    	if (other == null || !(other instanceof BaseField)) {
    		return -1;
    	}
        return statusByte - ((BaseField)other).statusByte;
    }

    public final boolean isNull() {
        return (statusByte & NULL_FIELD) != 0;
    }

    public final void setNull() {
        statusByte = NULL_FIELD;
    }

    public final boolean isNegativeInfinity() {
        return (statusByte & MINUS_INFINITY_FIELD) != 0;
    }

    public final boolean isPositiveInfinity() {
        return (statusByte & PLUS_INFINITY_FIELD) != 0;
    }

    public final void setNegativeInfinity() {
        statusByte = MINUS_INFINITY_FIELD;
    }

    public final void setPositiveInfinity() {
        statusByte = PLUS_INFINITY_FIELD;
    }

    public final boolean isValue() {
        return (statusByte & VALUE_FIELD) != 0;
    }

    public final void setValue() {
        statusByte = VALUE_FIELD;
    }

	@Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof BaseField)) {
            return false;
        }
        return compareTo((BaseField) o) == 0;
    }

	@Override
	public Object clone() throws CloneNotSupportedException {
		BaseField o = (BaseField) super.clone();
		o.statusByte = statusByte;
		o.typeDesc = typeDesc;
		return o;
	}

    public final TypeDescriptor getType() {
        return typeDesc;
    }

}
