/***
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Project: www.simpledbm.org
 *    Author : Dibyendu Majumdar
 *    Email  : dibyendu@mazumdar.demon.co.uk
 */
package org.simpledbm.common.registry;

/**
 * Specifies the interface for creating objects based upon
 * type codes. This is useful for re-creating objects that have been
 * retrieved from a persistent store. Also supports registration of
 * Singletons. 
 * 
 * @author Dibyendu Majumdar
 * @since 07-Aug-2005
 */
public interface ObjectFactory {

    /**
     * Registers a class to the Object Registry. 
     *  
     * @param typename A unique name for the type.
     * @param typecode A unique type code for the type.
     * @param classname The class name.
     */
    public void register(int typecode, String classname);    
    
    /**
     * Registers a Singleton object to the registry.
     * 
     * @param typename A unique name for the type.
     * @param typecode A unique type code for the type.
     * @param object The object to be registered.
     */
    public void register(int typecode, Object object);

	/**
	 * Creates an instance of the specified type.
	 * 
	 * @param typecode The code for the type
	 * @return Object of the specified type.
	 */
	Object getInstance(int typecode);
}
