/***
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Project: www.simpledbm.org
 *    Author : Dibyendu Majumdar
 *    Email  : dibyendu@mazumdar.demon.co.uk
 */
package org.simpledbm.rss.tm.impl;

import java.nio.ByteBuffer;

import org.simpledbm.common.registry.ObjectFactory;
import org.simpledbm.rss.log.LogRecord;
import org.simpledbm.rss.tm.BaseLoggable;
import org.simpledbm.rss.tm.Loggable;
import org.simpledbm.rss.tm.LoggableFactory;
import org.simpledbm.rss.tm.LoggableFactoryAware;

/**
 * A factory for creating {@link org.simpledbm.rss.tm.Loggable} objects.
 * 
 * @author Dibyendu Majumdar
 * @since 23-Aug-2005
 */
public final class LoggableFactoryImpl implements LoggableFactory {
	
	private final ObjectFactory objectFactory;
	
	public LoggableFactoryImpl(ObjectFactory objectFactory) {
		this.objectFactory = objectFactory;
	}
	
	public final Loggable getInstance(ByteBuffer bb) {
		bb.mark();
		short typecode = bb.getShort();
		bb.reset();
		BaseLoggable loggable = (BaseLoggable) objectFactory.getInstance(typecode);
		if (loggable instanceof LoggableFactoryAware) {
			((LoggableFactoryAware)loggable).setLoggableFactory(this);
		}
		loggable.init();
		loggable.retrieve(bb);
		return loggable;
	}
	
	public final Loggable getInstance(LogRecord logRec) {
		byte[] data = logRec.getData();
		ByteBuffer bb = ByteBuffer.wrap(data);
		Loggable loggable = getInstance(bb);
		loggable.setLsn(logRec.getLsn());
		// TODO loggable.setPrevLsn();
		return loggable;
	}

    public final Loggable getInstance(int moduleId, int typecode) {
        BaseLoggable loggable = (BaseLoggable) objectFactory.getInstance(typecode);
        loggable.setTypecode(typecode);
        loggable.setModuleId(moduleId);
		if (loggable instanceof LoggableFactoryAware) {
			((LoggableFactoryAware)loggable).setLoggableFactory(this);
		}
        loggable.init();
        return loggable;
    }

}
