/***
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Project: www.simpledbm.org
 *    Author : Dibyendu Majumdar
 *    Email  : dibyendu@mazumdar.demon.co.uk
 */
package org.simpledbm.rss.sm.impl;

import junit.framework.TestCase;

import org.simpledbm.common.latch.LatchFactory;
import org.simpledbm.common.latch.impl.LatchFactoryImpl;
import org.simpledbm.common.registry.ObjectFactory;
import org.simpledbm.common.registry.impl.ObjectFactoryImpl;
import org.simpledbm.rss.bm.BufMgr;
import org.simpledbm.rss.bm.impl.BufMgrImpl;
import org.simpledbm.rss.io.StorageContainer;
import org.simpledbm.rss.io.StorageContainerFactory;
import org.simpledbm.rss.io.StorageManager;
import org.simpledbm.rss.io.impl.FileStorageContainerFactory;
import org.simpledbm.rss.io.impl.StorageManagerImpl;
import org.simpledbm.rss.locking.LockMgr;
import org.simpledbm.rss.locking.LockMgrFactory;
import org.simpledbm.rss.locking.impl.LockMgrFactoryImpl;
import org.simpledbm.rss.log.LogMgr;
import org.simpledbm.rss.log.impl.LogFactoryImpl;
import org.simpledbm.rss.pm.Page;
import org.simpledbm.rss.pm.PageFactory;
import org.simpledbm.rss.pm.PageId;
import org.simpledbm.rss.pm.impl.PageFactoryImpl;
import org.simpledbm.rss.sm.SpaceChecker;
import org.simpledbm.rss.sm.SpaceMgrException;
import org.simpledbm.rss.sm.SpaceScan;
import org.simpledbm.rss.sm.impl.SpaceMgrImpl;
import org.simpledbm.rss.sm.impl.SpaceMgrImpl.SpaceCursorImpl;
import org.simpledbm.rss.tm.LoggableFactory;
import org.simpledbm.rss.tm.Savepoint;
import org.simpledbm.rss.tm.Transaction;
import org.simpledbm.rss.tm.TransactionalModuleRegistry;
import org.simpledbm.rss.tm.TrxMgr;
import org.simpledbm.rss.tm.impl.LoggableFactoryImpl;
import org.simpledbm.rss.tm.impl.TransactionalModuleRegistryImpl;
import org.simpledbm.rss.tm.impl.TrxMgrImpl;

public class SpaceMgrTests extends TestCase {

	public static void main(String[] args) {
	}

	public SpaceMgrTests(String arg0) {
		super(arg0);
	}

	public void testOneBitSpaceMapPage() throws Exception {

		/* Create the write ahead log */
		final LogFactoryImpl logFactory = new LogFactoryImpl();
		logFactory.createLog(null);

		final ObjectFactory objectFactory = new ObjectFactoryImpl();
		final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
		final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
		final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
		final LockMgr lockmgr = lockmgrFactory.create(null);
		final LogMgr logmgr = logFactory.getLog(null);
		logmgr.start();
		final BufMgr bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
		bufmgr.start();
		final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
		final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
		final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
		final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
    	StorageContainer sc = storageFactory.create("dual");
    	storageManager.register(0, sc);
        Page page = pageFactory.getInstance(pageFactory.getRawPageType(), new PageId(0, 0));
        pageFactory.store(page);

		try {
			SpaceMgrImpl.OneBitSpaceMapPage onebitsmp = (SpaceMgrImpl.OneBitSpaceMapPage) pageFactory.getInstance(SpaceMgrImpl.TYPE_ONEBITSPACEMAPPAGE, new PageId(1, 0));
			onebitsmp.setFirstPageNumber(0);
			assertEquals(onebitsmp.getCount(), 65328);
			for (int i = 0; i < onebitsmp.getCount(); i++) {
				assertTrue(onebitsmp.getSpaceBits(i) == 0);
			}
			onebitsmp.setSpaceBits(0, 1);
			onebitsmp.setSpaceBits(onebitsmp.getLastPageNumber(), 1);
			try {
				onebitsmp.setSpaceBits(onebitsmp.getCount(), 1);
				fail();
			} catch (SpaceMgrException e) {
			}
			assertTrue(onebitsmp.getSpaceBits(0) == 1);
			assertTrue(onebitsmp.getSpaceBits(onebitsmp.getLastPageNumber()) == 1);
			for (int i = 1; i < onebitsmp.getLastPageNumber(); i++) {
				assertTrue(onebitsmp.getSpaceBits(i) == 0);
			}

			SpaceMgrImpl.TwoBitSpaceMapPage twobitsmp = (SpaceMgrImpl.TwoBitSpaceMapPage) pageFactory.getInstance(SpaceMgrImpl.TYPE_TWOBITSPACEMAPPAGE, new PageId(1, 0));
			twobitsmp.setFirstPageNumber(0);
			for (int i = 0; i < twobitsmp.getCount(); i++) {
				assertTrue(twobitsmp.getSpaceBits(i) == 0);
			}
			twobitsmp.setSpaceBits(0, 2);
			twobitsmp.setSpaceBits(twobitsmp.getLastPageNumber(), 3);
			try {
				twobitsmp.setSpaceBits(twobitsmp.getCount(), 1);
				fail();
			} catch (SpaceMgrException e) {
			}
			assertTrue(twobitsmp.getSpaceBits(0) == 2);
			assertTrue(twobitsmp.getSpaceBits(twobitsmp.getLastPageNumber()) == 3);
			for (int i = 1; i < twobitsmp.getLastPageNumber(); i++) {
				assertTrue(twobitsmp.getSpaceBits(i) == 0);
			}

			assertEquals(twobitsmp.getCount(), 32664);
		} finally {
			trxmgr.shutdown();
			bufmgr.shutdown();
			logmgr.shutdown();
			storageManager.shutdown();
		}
	}

    public void doCreateContainer(boolean commit) throws Exception {

        final LogFactoryImpl logFactory = new LogFactoryImpl();
        final ObjectFactory objectFactory = new ObjectFactoryImpl();
        final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
        final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
        final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
        final LockMgr lockmgr = lockmgrFactory.create(null);
        final LogMgr logmgr = logFactory.getLog(null);
        logmgr.start();
        final BufMgr bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
        bufmgr.start();
        final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
        final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
        final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
        final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
        SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
        
        if (!commit) {
            spacemgr.setTesting(1);
        }
        try {
            StorageContainer sc = storageFactory.open("dual");
            storageManager.register(0, sc);

            trxmgr.start();
			
            Transaction trx = trxmgr.begin();
            boolean okay = false;
            try {
                spacemgr.createContainer(trx, "testctr.dat", 1, 1, 8, pageFactory.getRawPageType());
				assertTrue(storageManager.getInstance(1) != null);
                okay = true;
            }
            finally {
                if (okay && commit) {
                    trx.commit();
                }
                else {
                    trx.abort();
					assertTrue(storageManager.getInstance(1) == null);
                }
            }
        }
        finally {
			trxmgr.shutdown();
            bufmgr.shutdown();
            logmgr.shutdown();
            storageManager.shutdown();
        }
    }

    public void testCreateContainerAbort() throws Exception {
    	try {
    		doCreateContainer(false);
    	}
    	catch (SpaceMgrException.TestException e) {
    		// Expected
    	}
    }
    
	public void testCreateContainerCommit() throws Exception {
        doCreateContainer(true);
	}

    void doTestOpenAndDropContainer(boolean commit) throws Exception {

        final LogFactoryImpl logFactory = new LogFactoryImpl();
        final ObjectFactory objectFactory = new ObjectFactoryImpl();
        final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
        final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
        final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
        final LockMgr lockmgr = lockmgrFactory.create(null);
        final LogMgr logmgr = logFactory.getLog(null);
        logmgr.start();
        final BufMgr bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
        bufmgr.start();
        final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
        final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
        final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
        final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
        SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
        
        try {
            trxmgr.start();
            assertTrue(storageManager.getInstance(0) != null);
            assertTrue(storageManager.getInstance(1) != null);
            
            Transaction trx = trxmgr.begin();
            spacemgr.dropContainer(trx, 1);
            if (commit) {
                trx.commit();
                assertTrue(storageManager.getInstance(1) == null);
            }
            else {
                trx.abort();
                assertTrue(storageManager.getInstance(1) != null);
            }
        }
        finally {
			trxmgr.shutdown();
            bufmgr.shutdown();
            logmgr.shutdown();
            storageManager.shutdown();
        }
    }

    void doTestOpenAndDropContainerRollback() throws Exception {

        final LogFactoryImpl logFactory = new LogFactoryImpl();
        final ObjectFactory objectFactory = new ObjectFactoryImpl();
        final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
        final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
        final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
        final LockMgr lockmgr = lockmgrFactory.create(null);
        final LogMgr logmgr = logFactory.getLog(null);
        logmgr.start();
        final BufMgr bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
        bufmgr.start();
        final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
        final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
        final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
        final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
        SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
        
        try {
            trxmgr.start();
            assertTrue(storageManager.getInstance(0) != null);
            assertTrue(storageManager.getInstance(1) != null);
            
            Transaction trx = trxmgr.begin();
            Savepoint sp = trx.createSavepoint();
            spacemgr.dropContainer(trx, 1);
            trx.rollback(sp);
            trx.commit();
            assertTrue(storageManager.getInstance(1) != null);
        }
        finally {
			trxmgr.shutdown();
            bufmgr.shutdown();
            logmgr.shutdown();
            storageManager.shutdown();
        }
    }
    
    
	public void testDropContainerCommit() throws Exception {
        doTestOpenAndDropContainer(true);
	}

    public void testCreateContainerCommit2() throws Exception {
        doCreateContainer(true);
    }

    public void testDropContainerAbort() throws Exception {
        doTestOpenAndDropContainer(false);
    }

    public void testDropContainerRollback() throws Exception {
        doTestOpenAndDropContainerRollback();
    }
    
	public void testExtendContainer() throws Exception {

		final LogFactoryImpl logFactory = new LogFactoryImpl();
		final ObjectFactory objectFactory = new ObjectFactoryImpl();
		final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
		final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
		final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
		final LockMgr lockmgr = lockmgrFactory.create(null);
		final LogMgr logmgr = logFactory.getLog(null);
		logmgr.start();
		final BufMgr bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
		bufmgr.start();
		final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
		final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
		final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
		final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
    	SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
    	
		try {
	    	trxmgr.start();
			assertTrue(storageManager.getInstance(0) != null);
			assertTrue(storageManager.getInstance(1) != null);
			
	    	Transaction trx = trxmgr.begin();
	    	spacemgr.extendContainer(trx, 1);
	    	trx.abort();
		}
		finally {
			trxmgr.shutdown();
			bufmgr.shutdown();
			logmgr.shutdown();
			storageManager.shutdown();
		}
	}	
	
	public void testSpaceCursor() throws Exception {

		final LogFactoryImpl logFactory = new LogFactoryImpl();
		final ObjectFactory objectFactory = new ObjectFactoryImpl();
		final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
		final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
		final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
		final LockMgr lockmgr = lockmgrFactory.create(null);
		final LogMgr logmgr = logFactory.getLog(null);
		logmgr.start();
		final BufMgrImpl bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
		bufmgr.start();
		final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
		final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
		final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
		final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
    	SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
        bufmgr.setStorageManager(storageManager);
    	
		try {
	    	trxmgr.start();
			assertTrue(storageManager.getInstance(0) != null);
			assertTrue(storageManager.getInstance(1) != null);
	    
			int expectedPages[] = { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, -1 };
			
	    	SpaceCursorImpl spaceCursor = new SpaceCursorImpl(spacemgr, 1);
	    	for (int i = 0; i < (expectedPages.length + 5); i++) {
	    		int pageNumber = spaceCursor.findAndFixSpaceMapPageExclusively(new SpaceChecker() {
	    			public boolean hasSpace(int value) {
	    				return value == 0;
	    			}
	    		});
	    		assertTrue(i < expectedPages.length);
	    		assertEquals(expectedPages[i], pageNumber);
	    		if (pageNumber == -1) {
	    			break;
	    		}
	    		Transaction trx = trxmgr.begin();
	    		spaceCursor.updateAndLogRedoOnly(trx, pageNumber, 1);
	    		spaceCursor.unfixCurrentSpaceMapPage();
	    		trx.commit();
	    	}

			expectedPages = new int[] { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, -1 };
	    	
	    	SpaceScan spaceScan = spacemgr.openScan(1);
			try {
				int i = 0;
				while (spaceScan.fetchNext()) {
					int pageNumber = spaceScan.getCurrentPage();
		    		assertTrue(i < expectedPages.length);
		    		assertEquals(expectedPages[i], pageNumber);
					i++;
				}
				assertTrue(spaceScan.isEof());
			} finally {
				spaceScan.close();
			}
	    	
			Transaction trx = trxmgr.begin();
			spacemgr.dropContainer(trx, 1);
			trx.commit();
			assertTrue(storageManager.getInstance(1) == null);
		}
		finally {
			trxmgr.shutdown();
			bufmgr.shutdown();
			logmgr.shutdown();
			storageManager.shutdown();
		}
	}		

	public void testReopenContainer() throws Exception {

		final LogFactoryImpl logFactory = new LogFactoryImpl();
		final ObjectFactory objectFactory = new ObjectFactoryImpl();
		final StorageContainerFactory storageFactory = new FileStorageContainerFactory();
		final StorageManager storageManager = new StorageManagerImpl();
        final LatchFactory latchFactory = new LatchFactoryImpl();
        final PageFactory pageFactory = new PageFactoryImpl(objectFactory,
                storageManager, latchFactory);
		final LockMgrFactory lockmgrFactory = new LockMgrFactoryImpl();
		final LockMgr lockmgr = lockmgrFactory.create(null);
		final LogMgr logmgr = logFactory.getLog(null);
		logmgr.start();
		final BufMgrImpl bufmgr = new BufMgrImpl(logmgr, pageFactory, 3, 11);
		bufmgr.start();
		final LoggableFactory loggableFactory = new LoggableFactoryImpl(objectFactory);
		final TransactionalModuleRegistry moduleRegistry = new TransactionalModuleRegistryImpl();
		final TrxMgr trxmgr = new TrxMgrImpl(logmgr, storageFactory, storageManager, bufmgr, lockmgr, loggableFactory, latchFactory, objectFactory, moduleRegistry);
		final SpaceMgrImpl spacemgr = new SpaceMgrImpl(objectFactory, pageFactory, logmgr, bufmgr, storageManager, storageFactory, loggableFactory, trxmgr, moduleRegistry);
    	SpaceMgrImpl.SpaceMapPageImpl.TESTING = true;
        bufmgr.setStorageManager(storageManager);
    	
		try {
//	    	StorageContainer sc = storageFactory.open("dual");
//	    	storageManager.register(0, sc);

	    	trxmgr.start();
			assertTrue(storageManager.getInstance(0) != null);
			assertTrue(storageManager.getInstance(1) == null);
		}
		finally {
			trxmgr.shutdown();
			bufmgr.shutdown();
			logmgr.shutdown();
			storageManager.shutdown();
		}
	}
}
